/************************************************************************
* COPYRIGHT:   Copyright    1999 Symbol Technologies, Inc. 
*
* FILE:        Print Sample
*
* SYSTEM:      Symbol Print API for Palm III.
* 
*
* DESCRIPTION: Provides a sample application for the Symbol Print API.*
*
* HISTORY:     03/22/99    MS   Created
*              ...
*************************************************************************/

#include <Pilot.h>
#include <ptPrint.h>
#include "Print Sample.h"
#include "about.h"
#include "info.h"
#include "Print SampleRsc.h"

/***********************************************************************
 *
 *   Internal Constants
 *
 ***********************************************************************/
 
#define BuffSize				256

#define	sleep(x)				SysTaskDelay( (x) * sysTicksPerSecond )
#define FormGetValue(x)			CtlGetValue( GetObjectPtr ((x)) )
#define FormSetValue(x,y)		CtlSetValue( GetObjectPtr ((x)), (y) )

 
/***********************************************************************
 *
 *   Static Global variables
 *
 ***********************************************************************/
#pragma mark -- Global variables --

static	Word		gPrinterValue	= 0;
static	Boolean		gPrinterOpen	= false;

static  PTTransport	gTransport		= PTUnknown;

static	ULong		gQueryBufLength = BuffSize;
static	char		gQueryBuf[ BuffSize ];
static	char		gQueryBufRes[ BuffSize ];


static	PTConnectSettings gSettings	= { 9600, 
									  	PTDefaultSerTimeout, 
									  	PTDefaultSerFlags,
									  	PTDefaultSerRecvBuf
									  };

static	CharPtr clearLine = "                                                          ";

CharPtr ptStatusSting[] = {

	"Successful and complete",
	"Operation failed",
	"Rom Incompatible",	
	"Bad Parameter",	
	"Transport not Available",
	"Line Error",
	"Transport not Opened",
	"Port already open",
	"No Memory",
	"Printer already connected",		
	"Printer not found",	
	"Time out occurred",	
	"Ir Status Pending",
	"No Peer Addr",
	"Bind failed",
	"No Ir Device Found",
	"Ir Discovery Failed",
	"Ir IrConnectReq Failed",
	"Ir ConnectIrLap Failed",
	"Ir Query Failed",
	"Error Reading Printcap info",
	"                                        "
};

#define PTClear	PTStatusPrintCapFailed + 1

typedef enum {
	cmdOpen = 0,
	cmdClose,
	cmdForm,
	cmdData,
	cmdQueryPrinter,
	cmdInitPrinter,
	cmdResetPrinter,
	cmdPrinterVersion,
	cmdHighLevelAPI
} cmdCommands;


typedef enum {
	queryBaudRate = 0,
	queryStopBit,
	queryParity,
	queryQuery,
	queryInit,
	queryReset
} queryPrintCap;


/***********************************************************************
 *
 *  Shared Global variables
 *
 ***********************************************************************/

Char 	gInfoLabel[134];
CharPtr gInfoLabelPtr = &gInfoLabel[0];

Char 	gInfo[1024];
CharPtr gInfoPtr = &gInfo[0];

/***********************************************************************
 *
 *   Internal Functions
 *
 ***********************************************************************/
#pragma mark -- Static Functions --

static DWord 	StarterPilotMain(Word cmd, Ptr cmdPBP, Word launchFlags);
static Boolean	AppHandleEvent( EventPtr eventP);
static void 	AppEventLoop(void);
static Err 		AppStart(void);
static void 	AppStop(void);

static PTStatus Connect();
static Boolean	CheckFormReady();
static void		Disconnect();
static void		DoMessage( CharPtr prmMsg );
static CharPtr	GetPrinterModel();

static Boolean 	MainFormDoCommand(Word command);
static void 	MainFormInit( FormPtr frmP );
static Boolean 	MainFormHandleEvent(EventPtr eventP);

static void		PrinterClose();
static void 	PrinterInit();
static void 	PrinterOpen();
static void 	PrinterQuery();
static void 	PrinterReset();
static void		PrinterVersion();
static void		PrinterHighLevelAPI();

static void 	SendData( Word controlID );
static BytePtr 	SetData();
static BytePtr 	SetForm();
static void		SetPrinter( Word printerId );
static void 	SetStatus( Int status );
static void		SetTrigerLabel(Word list, Word trigger, short index );

/***********************************************************************
 *
 * FUNCTION:    DoMessage
 *
 * DESCRIPTION: Prints a line
 *
 * PARAMETERS:  prmMsg - message
 *
 * RETURNED: 
 *
 *
 ***********************************************************************/
 
static void DoMessage( CharPtr prmMsg )
{
    WinDrawChars( &clearLine[0], StrLen( &clearLine[0] ), 5, 130 );
    WinDrawChars( prmMsg, StrLen( prmMsg ), 5, 130 );
}

/***********************************************************************
 *
 * FUNCTION:    SetStatus
 *
 * DESCRIPTION: Prints a line
 *
 * PARAMETERS:  status - index to messages
 *
 * RETURNED: 
 *
 *
 ***********************************************************************/
 
static void SetStatus( Int status )
{
    WinDrawChars( &clearLine[0], StrLen( &clearLine[0] ), 40, 145 );
    WinDrawChars( ptStatusSting[status], StrLen(ptStatusSting[status]), 40, 145 );
}

/***********************************************************************
 *
 * FUNCTION:    MainFormDoCommand
 *
 * DESCRIPTION: This routine performs the menu command specified.
 *
 * PARAMETERS:  command  - menu item id
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static Boolean MainFormDoCommand(Word command)
{
	Boolean handled = false;

	switch (command)
	{
		case MainOptionsAboutMonarchPrintSample:
		{
			MenuEraseStatus( 0 );
			FrmPopupForm( AboutForm );
			handled = true;
			break;
		}
		
		default:
		{
			break;
		}	
	}

	return handled;
}

/***********************************************************************
 *
 * FUNCTION:    SetPrinter
 *
 * DESCRIPTION: Set the printer name
 *
 * PARAMETERS:  controlID - list item value
 *
 * RETURNED:  
 *
 *
 ***********************************************************************/
 
static void SetPrinter( Word controlID )
{
	
	if ( gPrinterOpen ) {
		
		if ( controlID == gPrinterValue ) {
		
			FormSetValue( controlID, true );
		}
		else {
			FormSetValue( controlID, false );
		}
		
		FrmAlert( PrinterOpenAlert);
		return;
	}
	
	gPrinterValue = 0;

	gPrinterValue = MainPalmPrinterCheckbox;
	
	if ( FormGetValue ( controlID ) ) {
		gPrinterValue = controlID;
	}
		
	FormSetValue( Main9490Checkbox, 
				( gPrinterValue == Main9490Checkbox ) );

	FormSetValue( MainPalmPrinterCheckbox, 
				( gPrinterValue == MainPalmPrinterCheckbox ) );	
}


/***********************************************************************
 *
 * FUNCTION:    CheckFormReady
 *
 * DESCRIPTION: Determine if all information is availble to open the printer.
 *
 * PARAMETERS:  
 *
 * RETURNED: true if printer and transport are selected. 
 *
 *
 ***********************************************************************/
 
static Boolean CheckFormReady()
{

	gPrinterValue = MainPalmPrinterCheckbox;

	if ( 0 == gPrinterValue ) {
	
		FrmAlert( NoPrinterSelectedAlert );
	}
	
	if ( PTUnknown == gPrinterValue ) {
	
		FrmAlert( NoTransportSelectedAlert );
	}	
	
	if ( ( 0 == gPrinterValue ) || ( PTUnknown == gTransport ) ) {
		return false;
	}
	
	
	if ( ( Main9490Checkbox == gPrinterValue ) &&
		 ( PTIr == gTransport ) ) {
		 
		FrmAlert( PrinterTansportNotSupportedAlert );
		 
		return false;
	}
	
	return true;
}

/***********************************************************************
 *
 * FUNCTION:    GetPrinterModel
 *
 * DESCRIPTION: Returns the printer model used in the open.
 *
 * PARAMETERS:  
 *
 * RETURNED: true if printer and transport are selected. 
 *
 *
 ***********************************************************************/
 
static CharPtr GetPrinterModel()
{
	switch( gPrinterValue )
	{
		case Main9490Checkbox:
		{
			return "9490";
			break;
		}
		
		case MainPalmPrinterCheckbox:
		{
			return "6015";
			break;
		}
		
		default:
		{
			return "unknown";
			break;
		}
	}
}

/***********************************************************************
 *
 * FUNCTION:    SetSettings
 *
 * DESCRIPTION: Returns a PTConnectSettings.
 *
 * PARAMETERS:  
 *
 * RETURNED: Returns a PTConnectSettings.
 *
 *
 ***********************************************************************/

static PTConnectSettingsPtr SetSettings()
{
	gSettings.baudRate	= 19200;
	gSettings.recvBufSize = BuffSize;
	gSettings.timeOut	= PTDefaultSerTimeout;
	gSettings.flags		= serDefaultSettings | serSettingsFlagCTSAutoM;
	
	return &gSettings;
}	


/***********************************************************************
 *
 * FUNCTION:    SetData
 *
 * DESCRIPTION: Sets the data for a given printer.
 *
 * PARAMETERS:  
 *
 * RETURNED: BytePtr to the data.
 *
 *
 ***********************************************************************/

static BytePtr SetData()
{
	switch( gPrinterValue )
	{
		case Main9490Checkbox:
		{
			return (BytePtr)"{B,1,N,1|E,0,0,1,1,0,1|" \
							" 1,\"1. Monarch 9490\"|" \
							" 2,\"5\"|" \
							" 3,\" 3495\"|" \
							" 4,\"2. Monarch 9403\"|" \
							" 5,\"4\"|" \
							" 6,\"  995\"|" \
							" 7,\"3. Monarch 6035\"|" \
							" 8,\"3\"|" \
							" 9,\" 4995\"|" \
							"10,\"4. Monarch 6030\"|" \
							"11,\"2\"|" \
							"12,\" 2995\"|" \
							"13,\"5. Monarch 9450\"|" \
							"14,\"1\"|" \
							"15,\"  995\"|" \
							"}";
							
			break;
		}
		
		case MainPalmPrinterCheckbox:
		{
		
			return (BytePtr)"\x1bP#\r\n\r\n" \
			//                "              Paxar/Monarch\n" \
			//                "             170 Monarch Ln.\n" \
			//                "          Miamisburg, OH 45342\n" \
			//                "          Phone: (937) 865-2123\r\n\r\n" \
			//                "              SALES RECEIPT\r\n\r\n" \
			//                " Description                Qty.    Total\n" \
			//                " 1. Monarch 9490            5        3495\n" \
			//                " 2. Monarch 9403            4         995\n" \
			//                " 3. Monarch 6035            3        4995\n" \
			//                " 4. Monarch 6030            2        2995\n" \
			//                " 5. Monarch 9450            1         995\n" \
			//                "                                    -----\n" \
  			//                "                       Total        13475\r\n\r\n" \
  			//                " AMEX 37xyz55xx315001\n" 


//  							  "\x1bP\x01" \
//  			                  " Code 39 - Speed 1\r\n\r\n\r\n" \
//  			                  "  \x1bz1\x05\x32" \
//  			                  "12345\n" \
//  							  "\x1bP\x06" \
//  			                  " Code 39 - Speed 6\r\n\r\n\r\n" \
//  			                  "  \x1bz1\x05\x32" \
//  			                  "12345\n" \
//  							  "\x1bP\x07" \
//  			                  " Code 39 - Speed 7\r\n\r\n\r\n" \
//  			                  "  \x1bz1\x05\x32" \
//  			                  "12345\n" ;


							  "\x1bP\x07" \
  			                  " Code 39\r\n\r\n\r\n" \
  			                  "  \x1bz1\x05\x32" \
  			                  "12345\n" \
  			                  "  \x1bZ1\x05\x32" \
  			                  "12345\n\x0c" \
  			                  " Code 128\r\n\r\n\r\n" \
  			                  "  \x1bz2\x08\x32\x88" \
  			                  "AB31234\n" \
  			                  "  \x1bZ2\x08\x32\x88" \
  			                  "AB31234\n\x0c" \
 			                  " I 2of5\r\n\r\n\r\n" \
  			                  "  \x1bz3\x08\x32" \
  			                  "12345678\n" \
  			                  "  \x1bZ3\x08\x32" \
  			                  "12345678\n\x0c" \
  			                  " UPC-A\r\n\r\n\r\n" \
  			                  "  \x01bz4\x0B\x32" \
  			                  "65432165432\n" \
  			                  "  \x01bZ4\x0B\x32" \
  			                  "65432165432\n\x0c" \
  			                  " UPC-E\r\n\r\n\r\n" \
  			                  "  \x01bz4\x07\x32" \
  			                  "6543210\n" \
  			                  "  \x01bZ4\x07\x32" \
  			                  "6543210\n\x0c" \
  			                  " EAN/JAN-13\r\n\r\n\r\n" \
  			                  "  \x01bz4\x0C\x32" \
  			                  "496580496580\n" \
  			                  "  \x01bZ4\x0C\x32" \
  			                  "496580496580\n\x0c" \
    		                  " EAN/JAN-8\r\n\r\n\r\n" \
  			                  "  \x01bz4\x08\x32" \
  			                  "65432106\n" \
  			                  "  \x01bZ4\x08\x32" \
  			                  "65432106\n\x0c" \
  			                  " Codabar\r\n\r\n\r\n" \
  			                  "  \x01bz5\x08\x32" \
  			                  "A123456T\n" \
  			                  "  \x01bZ5\x08\x32" \
  			                  "A123456T\n\x0c";

  			//                "04992903123451234567890123\n\x0c";


			//return (BytePtr)"\x01bP#\r\n\r\n" \
			//				  "		    Home Beauty Supplies\n" \
			//				  "              100 MyHome Road.\n" \
			//				  "            Anytown, US  54321\n" \
			//			      "		   Phone: (123) 555-1234\r\n\r\n" \
			//				  "              SALES RECEIPT\r\n\r\n" \
			//				  " Description                Qty.    Total\n" \
			//				  " 1. Lipstick                2       12.00\n" \
			//				  " 2. Polish                  2       14.00\n" \
			//		  		  " 3. Fragrance               1        9.50\n" \
			//		  		  " 4. Lotion                  1       10.50\n" \
			//		  		  "                                    -----\n" \
			//		          "                        Total       46.00\r\n\r\n" \
  			//	              " AMEX 37xyz55xx315001\n" \
  			//                " Exp. Date 10/01\r\n\r\n\r\n" \
  			//                "  \x01bz1\x006\x032" \
  			//                  "123456\n\x00c";
 

			break;
			
		}
		
		default:
		{
			return NULL;
			break;
		}
	}
}

/***********************************************************************
 *
 * FUNCTION:    SetForm
 *
 * DESCRIPTION: Sets the form information for a given printer.
 *
 * PARAMETERS:  
 *
 * RETURNED: BytePtr to the form.
 *
 *
 ***********************************************************************/
 
static BytePtr SetForm()
{
	switch( gPrinterValue )
	{
		
		case Main9490Checkbox:
		{
			
			return (BytePtr)"{F,1,A,R,E,600,400,\"1SAMPLE\"|" \
							"C,383,75,0,2,1,1,B,L,0,0,\"Paxar/Monarch\"|" \
							"C,376,72,0,2,1,1,B,L,0,0,\"170 Monarch Ln.\"|" \
							"C,369,61,0,2,1,1,B,L,0,0,\"Miamisburg, OH 45342\"|" \
							"C,360,59,0,2,1,1,B,L,0,0,\"Phone: (937) 865-2123\"|" \
							"C,336,75,0,2,1,1,B,L,0,0,\"SALES RECEIPT\"|" \
							"C,314,20,0,2,1,1,B,L,0,0,\"Description\"|" \
							"C,313,122,0,2,1,1,B,L,0,0,\"Qty.\"|" \
							"C,313,155,0,2,1,1,B,L,0,0,\"Total\"|" \
							"L,S,400,0,400,200,4,\"11111111\"|" \
							"L,S,7,200,402,200,4,\"11111111\"|" \
							"T,1,25,V,300,20,0,2,1,1,B,L,0,0|" \
							"T,2,5,V,300,122,0,2,1,1,B,L,0,0|" \
							"T,3,7,V,300,155,0,2,1,1,B,L,0,0|" \
							"T,4,25,V,285,20,0,2,1,1,B,L,0,0|" \
							"T,5,5,V,285,122,0,2,1,1,B,L,0,0|" \
							"T,6,7,V,285,155,0,2,1,1,B,L,0,0|" \
							"T,7,25,V,270,20,0,2,1,1,B,L,0,0|" \
							"T,8,5,V,270,122,0,2,1,1,B,L,0,0|" \
							"T,9,7,V,270,155,0,2,1,1,B,L,0,0|" \
							"T,10,25,V,255,20,0,2,1,1,B,L,0,0|" \
							"T,11,5,V,255,122,0,2,1,1,B,L,0,0|" \
							"T,12,7,V,255,155,0,2,1,1,B,L,0,0|" \
							"T,13,25,V,240,20,0,2,1,1,B,L,0,0|" \
							"T,14,5,V,240,122,0,2,1,1,B,L,0,0|" \
							"T,15,7,V,240,155,0,2,1,1,B,L,0,0|" \
							"C,225,155,0,2,1,1,B,L,0,0,\"-----\"|" \
							"C,210,100,0,2,1,1,B,L,0,0,\"Total\"|" \
							"C,210,155,0,2,1,1,B,L,0,0,\"13475\"|" \
							"C,175,20,0,2,1,1,B,L,0,0,\"AMEX 373725513315001\"|" \
							"C,160,20,0,2,1,1,B,L,0,0,\"Exp. Date 10/01\"|" \
							"B,20,6,F,13,39,4,7,50,8,L,0|" \
							"R,1,\"123456\"|" \
							"}";
			break;
		}
			
		case MainPalmPrinterCheckbox:
		{
		
			return (BytePtr)"\x018";
			break;
			
		}
		
		
		default:
		{
			return NULL;
			break;
		}
	}
}

/***********************************************************************
 *
 * FUNCTION:    DoPrintCapQuery
 *
 * DESCRIPTION: Query the Printcap information
 *
 * PARAMETERS:  event - current event
 *
 * RETURNED: Displays the results in a popup form.
 *
 *
 ***********************************************************************/

static void DoPrintCapQuery(EventPtr event)
{
    PTStatus	status;
   
	if ( ! gPrinterOpen ) {
		FrmAlert( PrinterNotOpenAlert);
		return;
	}
	
	gQueryBufLength = BuffSize;	
	MemSet( &gQueryBuf[0], BuffSize, 0x0 );
	MemSet( &gQueryBufRes[0], gQueryBufLength, 0x0 );
	
	
	switch(event->data.popSelect.selection)
	{
		case queryBaudRate:
		{
			gQueryBuf[0] = 'b';
			gQueryBuf[1] = 'r';
			break; 
		}
		
		case queryStopBit:
		{
			gQueryBuf[0] = 's';
			gQueryBuf[1] = 'b';
			break; 
		}
		
		case queryParity:
		{
			gQueryBuf[0] = 'p';
			gQueryBuf[1] = 'r';
			break; 
		}
		
		case queryQuery:
		{
			gQueryBuf[0] = 'q';
			gQueryBuf[1] = 's';
			break; 
		}
		
		case queryInit:
		{
			gQueryBuf[0] = 'i';
			gQueryBuf[1] = 's';
			break; 
		}
		
		case queryReset:
		{
			gQueryBuf[0] = 'r';
			gQueryBuf[1] = 's';
			break; 
		}
		
	}
	
//	DoMessage( "ptQueryPrintCap" );
	SetStatus( PTClear );
	
	MemSet( gInfoPtr, 1024, 0x0 ); 
	MemSet( gInfoLabelPtr, 134, 0x0 ); 
	
	status = ptQueryPrintCap( (VoidPtr)&gQueryBuf[0], (VoidPtr)&gQueryBufRes[0], &gQueryBufLength );
	SetStatus( status );
	
	MemMove( gInfoLabelPtr, &gQueryBuf[0], StrLen( &gQueryBuf[0] ) + 1 );
	MemMove( gInfoPtr, &gQueryBufRes[0], gQueryBufLength );
	
	MemSet( gQueryBuf, BuffSize, 0x0 );
	MemSet( gQueryBufRes, BuffSize, 0x0 );			
		
	FrmPopupForm( InfoForm );	
	
}

/***********************************************************************
 *
 * FUNCTION:    SetTrigerLabel
 *
 * DESCRIPTION: Sets to label to the popup list to last command selected.
 *
 * PARAMETERS:  list - form list id
 *				trigger - object name of the list
 *				index - index of item in list 
 *
 * RETURNED: 
 *
 ***********************************************************************/
 
static void SetTrigerLabel(Word list, Word trigger, short index )
{

	ListPtr		pList;
	CharPtr		pLabel = NULL;
	
	pList = GetObjectPtr( list );
	pLabel = LstGetSelectionText(  pList, index );
	CtlSetLabel ( GetObjectPtr ( trigger ), pLabel );
	
}

/***********************************************************************
 *
 * FUNCTION:    PrinterClose
 *
 * DESCRIPTION: Execute the command ptClosePrinter API call.
 *
 * PARAMETERS:  EventPtr event
 *
 * RETURNED: 
 *
 ***********************************************************************/

static void PrinterClose( )
{

	PTStatus	status;
		
//	DoMessage( "ptClosePrinter" );
	SetStatus( PTClear );
					
	status = ptClosePrinter();
	SetStatus( status );
			
	gPrinterOpen = false;
}

/***********************************************************************
 *
 * FUNCTION:    PrinterInit
 *
 * DESCRIPTION: Execute the command PrinterInit API call.
 *
 * PARAMETERS:  none
 *
 * RETURNED:    none
 *
 ***********************************************************************/

static void PrinterInit()
{
	PTStatus	status;

	status = Connect();
	if ( PTStatusOK != status ) {
		return;
	}
	
	/*
	 * Init the printer
	 */	
//	DoMessage( "ptInitPrinter" );
	SetStatus( PTClear );
	
	
	status = ptInitPrinter( NULL, 0 ); 
	SetStatus( PTClear );
							
	Disconnect();
	
}

/***********************************************************************
 *
 * FUNCTION:    PrinterOpen
 *
 * DESCRIPTION: Execute the command ptOpenPrinter API call.
 *
 * PARAMETERS:  
 *
 * RETURNED: 
 *
 ***********************************************************************/
static void PrinterOpen()
{
	PTConnectSettingsPtr pcSettings;
    PTStatus	status;

	if ( ! CheckFormReady() ) {
		return;
	}
	
//	DoMessage( "ptOpenPrinter" );
	SetStatus( PTClear );
	
	pcSettings = SetSettings();
	
	status = ptOpenPrinter( GetPrinterModel(), gTransport, pcSettings );
//	status = ptOpenPrinter( GetPrinterModel(), gTransport, NULL );
	
	if ( PTStatusOK != status ) {
		
		FrmAlert( OpenFailedAlert);
		
	}
	
	gPrinterOpen = true;
	
	SetStatus( status );

}


/***********************************************************************
 *
 * FUNCTION:    PrinterHighLevelAPI
 *
 * DESCRIPTION: Execute the command ptPrintApiVersion API call.
 *
 * PARAMETERS:  
 *
 * RETURNED: 
 *
 ***********************************************************************/
 
static void PrinterHighLevelAPI()
{
	PTStatus	status;
	
	UInt		i;
	ULong		length;
	
	char		pbuf[40];
	char		fontInfo[256];
	

	 if ( ! CheckFormReady() ) {
		return;
	}
	
//	DoMessage( "High Level API" );
	SetStatus( PTClear );
	
//	DoMessage( "ptStartPrintBuffer" );
	status = ptStartPrintBuffer( 256 );
	if ( PTStatusOK != status ) {
		SetStatus( status );
		return;
	}

//	DoMessage( "Text and Line" );
	for ( i = 1; i <= 4; i++ ) {
	
		StrPrintF( &pbuf[0], "%s : %i", "Loop index ", i );
		DoMessage( "ptTextToBuffer Loop" );
		status = ptTextToBuffer( 30 + ( 5 * i ), i * 30, &pbuf[0] );
		if ( PTStatusOK != status && PTStatusPrintCapFailed != status) {
			ptResetPrintBuffer();
			SetStatus( status );
			return;
		}

//		DoMessage( "ptLineToBuffer Loop" );
		status = ptLineToBuffer( 30 + ( 5 * i  ), 
						20 + ( i * 30 ), 
						i * 120, 
						20 + ( i * 30 ), 
						i );
		if ( PTStatusOK != status && PTStatusPrintCapFailed != status) {
			ptResetPrintBuffer();
			SetStatus( status );
			return;
		}
	}

//	DoMessage( "ptTextToBuffer 2" );
	status = ptTextToBuffer( 30, 240, "Next to last." );
	if ( PTStatusOK != status && PTStatusPrintCapFailed != status) {
			ptResetPrintBuffer();
			SetStatus( status );
			return;
	}

	
//	DoMessage( "ptRectToBuffer 1" );
	status = ptRectToBuffer( 20, 215, 200, 280, 4 );
	if ( PTStatusOK != status && PTStatusPrintCapFailed != status) {
			ptResetPrintBuffer();
			SetStatus( status );
			return;
	}

	
	length = 256;
	status = ptQueryPrintCap( "f1", &fontInfo[0], &length );
	if ( PTStatusOK == status ) {
		
		ptSetFont( &fontInfo[0] );
	}
	
//	DoMessage( "ptTextToBuffer 3" );
	status = ptTextToBuffer( 30, 300, "The last line." );
	if ( PTStatusOK != status && PTStatusPrintCapFailed != status) {
			ptResetPrintBuffer();
			SetStatus( status );
			return;
	}

	
//	DoMessage( "ptLineToBuffer 1" );
	status = ptLineToBuffer( 30, 340, 500, 340, 11 );
	if ( PTStatusOK != status && PTStatusPrintCapFailed != status) {
			ptResetPrintBuffer();
			SetStatus( status );
			return;
	}
		
//	DoMessage( "ptPrintPrintBuffer" );
	status = ptPrintPrintBuffer( NULL );
	if ( PTStatusOK != status ) {
			ptResetPrintBuffer();
	}
	SetStatus( status );

}

/***********************************************************************
 *
 * FUNCTION:    SendData
 *
 * DESCRIPTION: Write data (data or form) to the printer.
 *
 * PARAMETERS:  controlID - list item value
 *
 * RETURNED: 
 *
 ***********************************************************************/
 
static void SendData( Word controlID )
{
    BytePtr				 pData;
    PTStatus			 status;
    
    if ( ! CheckFormReady() ) {
		return;
	}
	
	/*
	 * Set the data
	 */
	switch ( controlID )
	{
		case cmdForm:
		{
			pData = SetForm();
			break;
		}
		
		case cmdData:
		{
			pData = SetData();
			break;
		}
		
		default:
		{
			pData = (BytePtr)"debug string                           ";
			break;
		}
	}

    if (!pData)
    {
//    	DoMessage( "Nothing2Send" );
//		sleep( 1 );
	    return;
    }
       	
	status = Connect();
	if ( PTStatusOK != status ) {
		return;
	}
	
//	sleep( 1 );
	
	/*
	 * Write the form to the printer
	 */	
//	DoMessage( "ptWritePrinter" );
	SetStatus( PTClear );
	
	status = ptWritePrinter( pData, StrLen( (CharPtr)pData ) );		
	SetStatus( status );

//	sleep(1);

	Disconnect();
}
	
/***********************************************************************
 *
 * FUNCTION:    Disconnect
 *
 * DESCRIPTION: Execute the command ptDisconnectPrinter() API call.
 *
 * PARAMETERS: 
 *
 * RETURNED: 
 *
 ***********************************************************************/
 	
static void Disconnect()
{	
	/*
	 * Disconnect
	 */
//	sleep(1);
//	DoMessage( "ptDisconnectPrinter");
	SetStatus(  PTClear );
	SetStatus( ptDisconnectPrinter() );					
}

/***********************************************************************
 *
 * FUNCTION:    Connect
 *
 * DESCRIPTION: Execute the command ptConnectPrinter() API call.
 *
 * PARAMETERS:  
 *
 * RETURNED: 	PTStatus result code
 *
 ***********************************************************************/
 			
static PTStatus Connect()
{

	PTStatus status;

//	DoMessage( " " );
	SetStatus( PTStatusOK );
	
	/*
	 * Connect
	 */
//	DoMessage( "ptConnectPrinter" );
	SetStatus( PTClear );
				
	status = ptConnectPrinter( NULL );
	SetStatus( status );
	
	return status;
		
}

/***********************************************************************
 *
 * FUNCTION:    MainFormInit
 *
 * DESCRIPTION: This routine initializes the MainForm form.
 *
 * PARAMETERS:  frm - pointer to the MainForm form.
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
static void MainFormInit(FormPtr frmP)
{
	gTransport = PTSerial;
}

/***********************************************************************
 *
 * FUNCTION:    MainFormHandleEvent
 *
 * DESCRIPTION: This routine is the event handler for the 
 *              "MainForm" of this application.
 *
 * PARAMETERS:  eventP  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/

static Boolean MainFormHandleEvent(EventPtr eventP)
{
    Boolean		handled = false;
    FormPtr		frmP;
   	
	switch (eventP->eType) 
	{
		case frmOpenEvent:
		{
			frmP = FrmGetActiveForm();
			
			MainFormInit( frmP );
			FrmDrawForm ( frmP );
			
//			DoMessage( "Printer Closed" );
			SetStatus( PTStatusOK );
					
			handled = true;
			break;
		}
		
		case popSelectEvent:
		{
			break;
		}
		
		case menuEvent:
		{
			return MainFormDoCommand(eventP->data.menu.itemID);
			handled = true;
			break;
		}

	
		case ctlSelectEvent:
		{
			switch( eventP->data.ctlSelect.controlID )
			{
			
				case MainPrintSampleButton:
	
	
						// FORCE SERIAL
						
						// Check to make sure a printer has been selected!
						
									
						PrinterOpen( );					
						SendData( cmdForm );
						SendData( cmdData );
						PrinterClose( );
						break;
						

				/*
				 * Printers
				 */
				case Main9490Checkbox:
				case MainPalmPrinterCheckbox:
				{
					SetPrinter( eventP->data.ctlSelect.controlID  );
					handled = true;
					break;
				}
				
				default:
				{
					break;
				}
			
			}
			
			break;
		}
		
    	default:
    	{
    		break;
    	}
		
    }     	
	
	return handled;
}


/***********************************************************************
 *
 * FUNCTION:    AppHandleEvent
 *
 * DESCRIPTION: This routine loads form resources and set the event
 *              handler for the form loaded.
 *
 * PARAMETERS:  event  - a pointer to an EventType structure
 *
 * RETURNED:    true if the event has handle and should not be passed
 *              to a higher level handler.
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static Boolean AppHandleEvent( EventPtr eventP)
{
	Word formId;
	FormPtr frmP;

	if (eventP->eType == frmLoadEvent) {
	
		/*
		 *	Load the form resource.
		 */
		formId = eventP->data.frmLoad.formID;
		frmP = FrmInitForm(formId);
		FrmSetActiveForm(frmP);

		/*
		 * Set the event handler for the form.  The handler of the currently
		 * active form is called by FrmHandleEvent each time is receives an
		 * event.
		 */ 
		switch (formId)
		{
			case MainForm:
			{
				FrmSetEventHandler(frmP, MainFormHandleEvent);
				break;
			}
				
			case AboutForm:
			{
				FrmSetEventHandler( frmP, AboutHandleEvent );
				break;
			}
			
			case InfoForm:
			{
				FrmSetEventHandler( frmP, InfoHandleEvent );
				break;
			}
			
			default:
			{
				break;

			}

			return true;
		}
	}
	
	return false;
}


/***********************************************************************
 *
 * FUNCTION:    AppEventLoop
 *
 * DESCRIPTION: This routine is the event loop for the application.  
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static void AppEventLoop(void)
{
	Word error;
	EventType event;


	do {
		EvtGetEvent(&event, evtWaitForever);
		
		
		if (! SysHandleEvent(&event))
			if (! MenuHandleEvent(0, &event, &error))
				if (! AppHandleEvent(&event))
					FrmDispatchEvent(&event);

	} while (event.eType != appStopEvent);
}


/***********************************************************************
 *
 * FUNCTION:     AppStart
 *
 * DESCRIPTION:  Get the current application's preferences.
 *
 * PARAMETERS:   nothing
 *
 * RETURNED:     Err value 0 if nothing went wrong
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static Err AppStart(void)
{	
   return 0;
}


/***********************************************************************
 *
 * FUNCTION:    AppStop
 *
 * DESCRIPTION: Save the current state of the application.
 *
 * PARAMETERS:  nothing
 *
 * RETURNED:    nothing
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static void AppStop(void)
{
  	ptClosePrinter();
}


/***********************************************************************
 *
 * FUNCTION:    StarterPilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 *
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
 
static DWord StarterPilotMain(Word cmd, Ptr cmdPBP, Word launchFlags)
{
	Err error;

	switch (cmd)
		{
		case sysAppLaunchCmdNormalLaunch:
			error = AppStart();
			if (error) 
				return error;
				
			FrmGotoForm(MainForm);
			AppEventLoop();
			AppStop();
			break;

		default:
			break;

		}
	
	return 0;
}

/***********************************************************************
 *
 * FUNCTION:    GetObjectPtr
 *
 * DESCRIPTION: Returns a pointer to an object in the current form.
 *
 * PARAMETERS:  objectID 
 *
 * RETURNED:    VoidPtr
 *
 * REVISION HISTORY:
 *
 ***********************************************************************/
 
VoidPtr GetObjectPtr(Word objectID)
{
	FormPtr frmP;

	frmP = FrmGetActiveForm();
	return ( FrmGetObjectPtr( frmP, FrmGetObjectIndex(frmP, objectID) ) );
}

/***********************************************************************
 *
 * FUNCTION:    PilotMain
 *
 * DESCRIPTION: This is the main entry point for the application.
 *
 * PARAMETERS:  cmd - word value specifying the launch code. 
 *              cmdPB - pointer to a structure that is associated with the launch code. 
 *              launchFlags -  word value providing extra information about the launch.
 * RETURNED:    Result of launch
 *
 * REVISION HISTORY:
 *
 *
 ***********************************************************************/
DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags)
{
    return StarterPilotMain(cmd, cmdPBP, launchFlags);
}

